@@ -18,7 +18,6 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <sys/types.h>
-#include <gcrypt.h>
 #include <limits.h>
 #include <errno.h>
 #include <string.h>
@@ -26,6 +25,10 @@
 #include <getopt.h>
 #include <glib.h>
 
+#ifndef O_NOATIME
+# define O_NOATIME 0
+#endif
+
 #define SHA_DIGEST_LENGTH 20
 
 #define __tostring(x) #x 
@@ -331,12 +334,10 @@ void load_db(const char* dbname){
 	fprintf(stdout,"DELETE %s %s %s\n",m->name, \
 		txtsha(m->hsha, tmpbuff_1), txtsha(m->bsha, tmpbuff_2))
 	
-// TODO XXX change the implementation, names are useless since
-// they are the same!
 #define COMMAND_REPLACE(m,n) \
-	fprintf(stdout, "REPLACE %s %s %s WITH %s %s %s\n",\
+	fprintf(stdout, "REPLACE %s %s %s WITH %s %s\n",\
 		m->name,txtsha(m->hsha,tmpbuff_1),txtsha(m->bsha,tmpbuff_2),\
-		n->name,txtsha(n->hsha,tmpbuff_3),txtsha(n->bsha,tmpbuff_4))
+		txtsha(n->hsha,tmpbuff_3),txtsha(n->bsha,tmpbuff_4))
 
 #define COMMAND_REPLACE_HEADER(m,n) \
 	fprintf(stdout, "REPLACEHEADER %s %s %s WITH %s\n",\
@@ -345,12 +346,12 @@ void load_db(const char* dbname){
 
 // the hearth 
 void analize_file(const char* dir,const char* file) {    
-	char *addr,*next;
+	unsigned char *addr,*next;
 	int fd, header_found;
 	struct stat sb;
-	//unsigned char* sha1;
 	struct mail* alias, *bodyalias, *m;
-	gcry_md_hd_t gctx;
+	GChecksum* ctx;
+	gsize ctx_len;
 
 	m = alloc_mail();
 	snprintf(next_name(), MAX_EMAIL_NAME_LEN,"%s/%s",dir,file);
@@ -404,17 +405,18 @@ void analize_file(const char* dir,const char* file) {
 	}
 
 	// calculate sha1
-	gcry_md_open(&gctx, GCRY_MD_SHA1, 0);
-	gcry_md_write(gctx, addr, next - addr);
-	gcry_md_final(gctx);
-	memcpy(m->hsha, gcry_md_read(gctx, 0), SHA_DIGEST_LENGTH);
-	gcry_md_close(gctx);
-	gcry_md_open(&gctx, GCRY_MD_SHA1, 0);
-	gcry_md_write(gctx, next, sb.st_size - (next - addr));
-	gcry_md_final(gctx);
-	memcpy(m->bsha, gcry_md_read(gctx, 0), SHA_DIGEST_LENGTH);
-	gcry_md_close(gctx);
-	
+	ctx = g_checksum_new(G_CHECKSUM_SHA1);
+	ctx_len = SHA_DIGEST_LENGTH;
+	g_checksum_update(ctx, addr, next - addr);
+	g_checksum_get_digest(ctx, m->hsha, &ctx_len);
+	g_checksum_free(ctx);
+
+	ctx = g_checksum_new(G_CHECKSUM_SHA1);
+	ctx_len = SHA_DIGEST_LENGTH;
+	g_checksum_update(ctx, next, sb.st_size - (next - addr));
+	g_checksum_get_digest(ctx, m->bsha, &ctx_len);
+	g_checksum_free(ctx);
+
 	munmap(addr, sb.st_size);
 	close(fd);
 
@@ -477,12 +479,19 @@ void analize_dir(const char* path){
 
 	while ( (dir_entry = readdir(dir)) != NULL) {
 		if (DT_REG == dir_entry->d_type){
+#ifdef __GLIBC__ 
 			const char* bname = basename(path);	
+#else
+			gchar* bname = g_path_get_basename(path);	
+#endif
 			if ( !strcmp(bname,"cur") || !strcmp(bname,"new"))
 				analize_file(path,dir_entry->d_name);
 			else
 				VERBOSE(analize_dir,"skipping '%s/%s', outside maildir\n",
 					path,dir_entry->d_name);
+#ifndef __GLIBC__ 
+			g_free(bname);
+#endif
 		} else if (DT_DIR == dir_entry->d_type && 
 				strcmp(dir_entry->d_name,"tmp") &&
 				strcmp(dir_entry->d_name,".") &&
@@ -523,11 +532,10 @@ void generate_deletions(){
 }
 
 int extra_sha_file(const char* file) {    
-	char *addr,*next;
+	unsigned char *addr,*next;
 	int fd, header_found;
 	struct stat sb;
-	//unsigned char* sha1;
-	gcry_md_hd_t gctx;
+	gchar* sha1;
 
 	fd = open(file, O_RDONLY | O_NOATIME);
 	if (fd == -1) {
@@ -565,16 +573,13 @@ int extra_sha_file(const char* file) {
 	}
 
 	// calculate sha1
-	gcry_md_open(&gctx, GCRY_MD_SHA1, 0);
-	gcry_md_write(gctx, addr, next - addr);
-	gcry_md_final(gctx);
-	fprintf(stdout, "%s ", txtsha(gcry_md_read(gctx, 0),tmpbuff_1));
-	gcry_md_close(gctx);
-	gcry_md_open(&gctx, GCRY_MD_SHA1, 0);
-	gcry_md_write(gctx, next, sb.st_size - (next - addr));
-	gcry_md_final(gctx);
-	fprintf(stdout, "%s\n", txtsha(gcry_md_read(gctx, 0),tmpbuff_1));
-	gcry_md_close(gctx);
+	fprintf(stdout, "%s ", 
+		sha1 = g_compute_checksum_for_data(G_CHECKSUM_SHA1, addr, next - addr));
+	g_free(sha1);
+	fprintf(stdout, "%s\n", 
+		sha1 = g_compute_checksum_for_data(G_CHECKSUM_SHA1, 
+				next, sb.st_size - (next - addr)));
+	g_free(sha1);
 	
 	munmap(addr, sb.st_size);
 	close(fd);
@@ -634,8 +639,7 @@ const char* long_options_doc[] = {
 // print help and bail out
 void help(char* argv0, int rc){
 	int i;
-	char *bname = strdup(argv0);
-	bname = basename(bname);
+	char *bname = g_path_get_basename(argv0);
 
 	fprintf(stdout,"\nUsage: %s [options] paths...\n",bname);
 	for (i=0;long_options[i].name != NULL;i++) {
@@ -646,10 +650,11 @@ void help(char* argv0, int rc){
 If paths is a list of regular files, %s outputs the sha1 of its header\n\
 and body separated by space.\n\n\
 If paths is a list of directories, %s outputs a list of actions a client\n\
-has to perform to syncronize a copy of the same maildir. This set of actions\n\
+has to perform to syncronize a copy of the same maildirs. This set of actions\n\
 is relative to a previous status of the maildir stored in the db file.\n\
 The input directories are traversed recursively, and every file encountered\n\
-is a potential mail message (if it contains no \\n\\n it is skipped).\n\n\
+inside directories named cur/ and new/ is a potential mail message (if it\n\
+contains no \\n\\n it is skipped).\n\n\
 Regular files and directories cannot be mixed in paths.\n\n\
 Every client must use a different db-file, and the db-file is strictly\n\
 related with the set of directories given as arguments, and should not\n\
@@ -657,7 +662,8 @@ be used with a different directory set. Adding items to the directory\n\
 set is safe, while removing them may not do what you want (delete actions\n\
 are generated).\n", bname, bname);
 	fprintf(stdout,
-		"\n© 2009 Enrico Tassi, released under GPLv3, no waranties\n\n");
+		"\nVersion %s, © 2009 Enrico Tassi, released under GPLv3, \
+no waranties\n\n",tostring(VERSION));
 	exit(rc);
 }
 
@@ -671,11 +677,7 @@ int main(int argc, char *argv[]) {
 	int option_index = 0;
 	time_t bigbang;
 
-  	if (!gcry_check_version (GCRYPT_VERSION)) {
-		fputs ("libgcrypt version mismatch\n", stderr);
-		exit (2);
-    }
-
+	glib_check_version(2,16,0);
 
 	for(;;) {
 		c = getopt_long(argc, argv, "vh", long_options, &option_index);
