@@ -1,6 +1,8 @@
 -- Released under the terms of GPLv3 or at your option any later version.
 -- No warranties.
 -- Copyright 2009 Enrico Tassi <gares@fettunta.org>
+--
+-- common code for smd-client/server
 
 local PROTOCOL_VERSION="1.0"
 
@@ -14,15 +16,33 @@ local __G = _G
 
 module('syncmaildir',package.seeall)
 
+-- set mddiff path
 MDDIFF = ""
 if string.sub(PREFIX,1,1) == '@' then
-		MDDIFF = os.getenv('HOME')..'/SYNC/syncmaildir/mddiff '
+		MDDIFF = os.getenv('HOME')..'/Projects/syncmaildir/mddiff '
 		io.stderr:write('smd-client not installed, assuming mddiff is: ',
 			MDDIFF,'\n')
 else
 		MDDIFF = PREFIX .. '/bin/mddiff '
 end
 
+-- set sha1sum executable name
+SHA1SUM = '@SHA1SUM@'
+if string.sub(SHA1SUM,1,1) == '@' then
+		SHA1SUM = 'sha1sum'
+end
+
+-- set xdelta executable name
+XDELTA = '@XDELTA@'
+if string.sub(XDELTA,1,1) == '@' then
+		XDELTA = 'xdelta'
+end
+
+CPN = '@CPN@'
+if string.sub(CPN,1,1) == '@' then
+		CPN = 'cp -n'
+end
+
 function set_verbose(v)
 	verbose = v
 end
@@ -37,6 +57,25 @@ function log_error(msg)
 	io.stderr:write('ERROR: ',msg,'\n')
 end
 
+function log_tag(tag)
+	io.stderr:write('TAGS: ',tag,'\n')
+end
+
+function log_tags(context, cause, human, ...)
+	if human then human = "necessary" else human = "avoidable" end
+	local suggestions = {}
+	local suggestions_string = ""
+	if select('#',...) > 0 then 
+			suggestions_string = 
+				"suggested-actions("..table.concat({...}," ")..")"
+	else 
+			suggestions_string = "" 
+	end
+	log_tag("error::context("..context..") "..
+		"probable-cause("..cause..") "..
+		"human-intervention("..human..")".. suggestions_string)
+end
+
 function trace(x)
 	if verbose then
 		local t = {}
@@ -58,6 +97,7 @@ function transmit(out, path, what)
 	if not f then
 		log_error("Unable to open "..path..": "..(err or "no error"))
 		log_error("The problem should be transient, please retry.")
+		log_tags("transmit", "simultaneous-mailbox-edit",false,"retry")
 		error('Unable to open requested file.')
 	end
 	local size, err = f:seek("end")
@@ -65,6 +105,8 @@ function transmit(out, path, what)
 		log_error("Unable to calculate the size of "..path)
 		log_error("If it is not a regular file, please move it away.")
 		log_error("If it is a regular file, please report the problem.")
+		log_tags("transmit", "non-regular-file",true,
+			"display-permissions("..quote(path)..")")
 		error('Unable to calculate the size of the requested file.')
 	end
 	f:seek("set")
@@ -111,7 +153,9 @@ function receive(inf,outfile)
 			log_error("Unable to open "..outfile.." for writing.")
 			log_error('It may be caused by bad directory permissions, '..
 				'please check.')
-			os.exit(1)
+			log_tags("receive", "non-writeable-file",true,
+				"display-permissions("..quote(outfile)..")")
+			error("Unable to write incoming data")
 	end
 
 	local line = inf:read("*l")
@@ -135,7 +179,7 @@ function handshake(dbfile)
 	-- send the protocol version and the dbfile sha1 sum
 	io.write('protocol ',PROTOCOL_VERSION,'\n')
 	touch(dbfile)
-	local inf = io.popen('sha1sum '.. dbfile,'r')
+	local inf = io.popen(SHA1SUM..' '.. dbfile,'r')
 	local db_sha = inf:read('*a'):match('^(%S+)')
 	io.write('dbfile ',db_sha,'\n')
 	io.flush()
@@ -149,25 +193,30 @@ function handshake(dbfile)
 		log_error("Hint: did you correctly setup the SERVERNAME variable")
 		log_error("on your client? Did you add an entry for it in your ssh")
 		log_error("configuration file?")
-		os.exit(1)
+		log_tags("handshake", "network-error",false,"retry")
+		error('Network error')
 	end
 	local protocol = line:match('^protocol (.+)$')
 	if protocol ~= PROTOCOL_VERSION then
 		log_error('Wrong protocol version.')
-		log_error('The same version of syncmaildir must be user on '..
+		log_error('The same version of syncmaildir must be used on '..
 			'both endpoints')
-		os.exit(1)
+		log_tags("handshake", "protocol-mismatch",true)
+		error('Protocol version mismatch')
 	end
 	line = io.read('*l')
 	if line == nil then
 		log_error "The client disconnected during handshake"
-		os.exit(1)
+		log_tags("handshake", "network-error",false,"retry")
+		error('Network error')
 	end
 	local sha = line:match('^dbfile (%S+)$')
 	if sha ~= db_sha then
 		log_error('Local dbfile and remote db file differ.')
 		log_error('Remove both files and push/pull again.')
-		os.exit(1)
+		log_tags("handshake", "db-mismatch",true,"run(rm "..
+			quote(dbfile)..")")
+		error('Database mismatch')
 	end
 end
 
@@ -191,7 +240,9 @@ function mkdir_p(path)
 				log_error("Unable to create directory "..dir)
 				log_error('It may be caused by bad directory permissions, '..
 					'please check.')
-				os.exit(1)
+				log_tags("mkdir", "wrong-permissions",true,
+					"display-permissions("..quote(dir)..")")
+				error("Directory creation failed")
 			end
 			mkdir_p_cache[dir] = true
 		end
@@ -206,10 +257,17 @@ function mkdir_p(path)
 end
 
 function tmp_for(path,use_tmp)
-	local use_tmp = use_tmp or true
+	if use_tmp == nil then use_tmp = true end
 	local t = {} 
+	local absolute = ""
+	if string.byte(path,1) == string.byte('/',1) then absolute = '/' end
 	for m in path:gmatch('([^/]+)') do t[#t+1] = m end
 	local fname = t[#t]
+	local time, pid, host, tags = fname:match('^(%d+)%.(%d+)%.([^:]+)(.*)$')
+	time = time or os.date("%s")
+	pid = pid or "1"
+	host = host or "localhost"
+	tags = tags or ""
 	table.remove(t,#t)
 	local i, found = 0, false
 	if use_tmp then
@@ -223,13 +281,26 @@ function tmp_for(path,use_tmp)
 	end
 	local newpath
 	if not found then
-		newpath = path .. '.new'
+		time = os.date("%s")
+		t[#t+1] = time..'.'..pid..'.'..host..tags
+		newpath = absolute .. table.concat(t,'/') 
 	else
 		t[#t+1] = fname
-		newpath = table.concat(t,'/') 
+		newpath = absolute .. table.concat(t,'/') 
 	end
 	mkdir_p(newpath)
-	while exists(newpath) do newpath = newpath .. 'x' end
+	local attempts = 0
+	while exists(newpath) do 
+		if attempts > 10 then
+			error('unable to generate a fresh tmp name')			
+		else 
+			time = os.date("%s")
+			host = host .. 'x'
+			t[#t] = time..'.'..pid..'.'..host..tags
+			newpath = absolute .. table.concat(t,'/') 
+			attempts = attempts + 1
+		end
+	end
 	return newpath
 end
 
@@ -263,8 +334,10 @@ function touch(f)
 	if h == nil then
 		h = io.open(f,'w')
 		if h == nil then
-			log_error('Unable to touch '..f)
-			os.exit(1)
+			log_error('Unable to touch '..quote(f))
+			log_tags("touch","bad-permissions",true,
+				"display-permissions("..quote(f)..")")
+			error("Unable to touch a file")
 		else
 			h:close()
 		end
@@ -273,6 +346,18 @@ function touch(f)
 	end
 end
 
+function quote(s)
+	return '"' .. s:gsub('"','\\"'):gsub("%)","\\)").. '"'
+end
+
+function homefy(s)
+	if string.byte(s,1) == string.byte('/',1) then
+		return s
+	else
+		return os.getenv('HOME')..'/'..s
+	end
+end	
+
 function set_strict()
 -- strict access to the global environment
 	setmetatable(__G,{
