@@ -26,13 +26,13 @@ prepare(){
 	rm -rf test.$N
 	mkdir test.$N
 	make --quiet
-	make --quiet install PREFIX=$ROOT/test.$N
+	make --quiet install-bin PREFIX=$ROOT/test.$N
 	
 	cd test.$N
 	export HOMEC=$PWD/target
 	export HOMES=$PWD
 	export PATH=$PWD/bin:$ORIGPATH
-	export LUA_INIT="package.path='$ROOT/test.$N/share/lua/5.1/?.lua;'..package.path"
+	export LUA_INIT="package.path='$ROOT/test.$N/share/lua/5.1/?.lua;'"
 	tar -xzf ../misc/Mail.testcase.tgz
 	
 	mkfifo s2c
